# MCollective Puppet Enterprise
#
# This module manages the initial configuration of MCollective for use with
# Puppet Enterprise.  The primary purpose is to generate RSA key pairs for the
# initial "peadmin" user account on the Puppet Master and a shared set of RSA keys
# for all of the MCollective server processes (Puppet Agent roles).
#
# Resources managed on a Puppet Master:
#
#  * peadmin user account
#  * RSA keys to identify and authorize the peadmin user.
#  * One set of RSA keys generated for shared use among all MCollective agents.
#
# Resources managed on a Puppet Agent:
#
#  * RSA keys for MCollective service
#  * peadmin user account public RSA key to authenticate the peadmin user
#
# == Parameters
#
# This class expects four facts to be provided:
#
#  * fact_is_puppetmaster (true/false)
#  * fact_is_puppetagent  (true/false)
#  * fact_stomp_server    (hostname)
#  * fact_stomp_port      (TCP port number)
#
# The facts are automatically set by the Puppet Enterprise installer for each
# system.
#
# This class defaults to the "psk" security provider.  You may
# optionally specify the "aespe_security" security provider by setting the
# $::mcollective_security_provider variable in the Puppet Dashboard or as a
# custom fact.
#
# If a pre-shared-key is used, a randomly generated string (The contents of the
# mcollective credientials file) with be fed into the SHA1 hash algorithm to
# produce a unique key.
class pe_mcollective {
  validate_re($::fact_is_puppetmaster, '^true$|^false$')
  validate_re($::fact_is_puppetconsole, '^true$|^false$')
  validate_re($::fact_is_puppetagent, '^true$|^false$')
  validate_re($::fact_stomp_server, '^[a-zA-Z0-9.-]+$')
  validate_re($::fact_stomp_port, '^[0-9]+$')

  # Stomp SSL Support
  # Turned on by default.
  $mcollective_enable_stomp_ssl = $::mcollective_enable_stomp_ssl ? {
    'true'  => true,
    'false' => false,
    undef   => true,
    ''      => true,
    default => $::mcollective_enable_stomp_ssl,
  }
  validate_bool($mcollective_enable_stomp_ssl)
  # This variable is used by the ActiveMQ template.
  $stomp_activemq_protocol = $mcollective_enable_stomp_ssl ? {
    true  => 'stomp+ssl',
    false => 'stomp+nio',
  }

  # The default mcollective security provider.  Change this variable if the
  # default changes as a result of Product's decision.
  $mcollective_security_provider_default = 'psk'

  # Convert facter strings to booleans
  $is_puppetmaster = $::fact_is_puppetmaster ? { 'true'  => true, 'false' => false }
  $is_puppetconsole = $::fact_is_puppetconsole ? { 'true'  => true, 'false' => false }
  $is_puppetagent = $::fact_is_puppetagent ? { 'true'  => true, 'false' => false }

  $stomp_server = $::fact_stomp_server ? {
    undef   => 'puppetmaster',
    default => $::fact_stomp_server,
  }
  $stomp_port = $::fact_stomp_port ? {
    undef   => '61613',
    default => $::fact_stomp_port,
  }
  # Variables used by ERB templates.  This may be dynamically generated in the future.
  $stomp_user                   = 'mcollective'
  $stomp_password_file_contents = file('/etc/puppetlabs/mcollective/credentials')
  # Only take the first line as the password. Editors leave trailing newlines.
  $stomp_password_array         = split($stomp_password_file_contents, '\n')
  $stomp_password               = $stomp_password_array[0]

  # Pre-Shared Key for MCollective
  $mcollective_psk_string       = sha1($stomp_password_file_contents)
  # Determine the MCollective Security Model
  $mcollective_security_provider  = $::mcollective_security_provider ? {
    undef   => $mcollective_security_provider_default,
    default => $::mcollective_security_provider,
  }
  # We only support two security providers for the time being.
  validate_re($mcollective_security_provider, '^psk$|^aespe_security$')

  if $is_puppetmaster {
    # Make sure ActiveMQ is running on the master.
    service { 'pe-activemq':
      ensure     => running,
      enable     => true,
      hasstatus  => true,
      hasrestart => true,
      require => [ File['/etc/puppetlabs/activemq/activemq.xml'] ],
    }
    # Make sure activemq is aware of the stomp password
    file { "/etc/puppetlabs/activemq/activemq.xml":
      ensure  => file,
      content => template("${module_name}/activemq.xml.erb"),
      owner   => '0',
      group   => 'pe-activemq',
      mode    => '0640',
      notify => [ Service['pe-activemq'] ],
    }
    # Configure the ActiveMQ Wrapper in case we've enabled SSL
    file { "/etc/puppetlabs/activemq/activemq-wrapper.conf":
      ensure  => file,
      content => template("${module_name}/activemq-wrapper.conf.erb"),
      owner   => 0,
      group   => 0,
      mode    => 0644,
      notify  => [ Service['pe-activemq'] ],
    }
    file { "credentials":
      path    => '/etc/puppetlabs/mcollective/credentials',
      owner   => 'pe-puppet',
      group   => 'pe-puppet',
      mode    => '0600',
    }

    # Make sure we have a shared certificate for all of the MC servers
    exec { 'mcollective-server-cert':
      command => 'puppet cert --generate pe-internal-mcollective-servers',
      path    => '/opt/puppet/bin:/usr/kerberos/sbin:/usr/kerberos/bin:/usr/local/sbin:/usr/local/bin:/sbin:/bin:/usr/sbin:/usr/bin',
      creates => '/etc/puppetlabs/puppet/ssl/certs/pe-internal-mcollective-servers.pem',
      before  => [ File['mcollective-public.pem'], File['mcollective-private.pem'], File['mcollective-cert.pem'] ],
    }
    exec { 'mcollective-client-cert':
      command => 'puppet cert --generate pe-internal-peadmin-mcollective-client',
      path    => '/opt/puppet/bin:/usr/kerberos/sbin:/usr/kerberos/bin:/usr/local/sbin:/usr/local/bin:/sbin:/bin:/usr/sbin:/usr/bin',
      creates => '/etc/puppetlabs/puppet/ssl/certs/pe-internal-peadmin-mcollective-client.pem',
      before  => [ File['peadmin-public.pem'] ]
    }
    exec { 'puppet-dashboard-client-cert':
      command => 'puppet cert --generate pe-internal-puppet-console-mcollective-client',
      path    => '/opt/puppet/bin:/usr/kerberos/sbin:/usr/kerberos/bin:/usr/local/sbin:/usr/local/bin:/sbin:/bin:/usr/sbin:/usr/bin',
      creates => '/etc/puppetlabs/puppet/ssl/certs/pe-internal-puppet-console-mcollective-client.pem',
      before  => [ File['puppet-dashboard-public.pem'] ]
    }

    # Generate the SSL Certificate used by the Stomp Server if SSL is enabled.
    if $mcollective_enable_stomp_ssl {
      # We use Puppet's CA to generate the private key and issue the certificate
      exec { 'broker_cert':
        command => "puppet cert --generate pe-internal-broker --dns_alt_names '${stomp_server},${fqdn},stomp'",
        path    => '/opt/puppet/bin:/usr/kerberos/sbin:/usr/kerberos/bin:/usr/local/sbin:/usr/local/bin:/sbin:/bin:/usr/sbin:/usr/bin',
        creates => '/etc/puppetlabs/puppet/ssl/certs/pe-internal-broker.pem',
        notify  => [ Exec['broker_cert_bundle'] ],
      }
      # We need to combine the artifacts in order to covert to PKCS12
      exec { 'broker_cert_bundle':
        command => "sh -c 'umask 077; cat /etc/puppetlabs/puppet/ssl/private_keys/pe-internal-broker.pem /etc/puppetlabs/puppet/ssl/certs/ca.pem /etc/puppetlabs/puppet/ssl/certs/pe-internal-broker.pem > /etc/puppetlabs/activemq/broker.pem'",
        path    => '/opt/puppet/bin:/usr/kerberos/sbin:/usr/kerberos/bin:/usr/local/sbin:/usr/local/bin:/sbin:/bin:/usr/sbin:/usr/bin',
        creates => '/etc/puppetlabs/activemq/broker.pem',
        before  => [ File['/etc/puppetlabs/activemq/broker.pem'] ],
      }
      file { '/etc/puppetlabs/activemq/broker.pem':
        ensure => file,
        owner  => 0,
        group  => 0,
        mode   => 600,
      }
      # Now convert the bundle to a PKCS12 format file
      exec { 'broker_cert_pkcs12':
        command => "sh -c 'umask 077; openssl pkcs12 -export -in broker.pem -out broker.p12 -name ${stomp_server} -passout pass:puppet'",
        cwd     => '/etc/puppetlabs/activemq',
        path    => '/opt/puppet/bin:/usr/kerberos/sbin:/usr/kerberos/bin:/usr/local/sbin:/usr/local/bin:/sbin:/bin:/usr/sbin:/usr/bin',
        creates => '/etc/puppetlabs/activemq/broker.p12',
        require => [ File['/etc/puppetlabs/activemq/broker.pem'] ],
        before  => [ File['/etc/puppetlabs/activemq/broker.p12'] ],
      }
      file { '/etc/puppetlabs/activemq/broker.p12':
        ensure => file,
        owner  => 0,
        group  => 0,
        mode   => 600,
      }
      # Now convert the PKCS12 format file into a Java Key Store
      exec { 'broker_cert_keystore':
        command => "sh -c 'umask 077; keytool -importkeystore -deststorepass puppet -destkeypass puppet -destkeystore broker.ks -srckeystore broker.p12 -srcstorepass puppet -srcstoretype PKCS12 -alias ${stomp_server}'",
        cwd     => '/etc/puppetlabs/activemq',
        path    => '/opt/puppet/bin:/usr/kerberos/sbin:/usr/kerberos/bin:/usr/local/sbin:/usr/local/bin:/sbin:/bin:/usr/sbin:/usr/bin',
        creates => '/etc/puppetlabs/activemq/broker.ks',
        require => [ File['/etc/puppetlabs/activemq/broker.p12'] ],
        before  => [ File['/etc/puppetlabs/activemq/broker.ks'] ],
      }
      file { '/etc/puppetlabs/activemq/broker.ks':
        ensure => file,
        owner   => '0',
        group   => 'pe-activemq',
        mode    => '0640',
        notify => [ Service['pe-activemq'] ],
      }
      # Finally, create the Java TrustStore file.  This isn't required on some JVM's, but it
      # is with the IBM JVM.  If the file does not exist ActiveMQ will fail to start.
      exec { 'broker_cert_truststore':
        command => "sh -c 'umask 077; keytool -import -noprompt -trustcacerts -alias 'PuppetCA' -file /etc/puppetlabs/puppet/ssl/certs/ca.pem -keystore broker.ts -storepass puppet'",
        cwd     => '/etc/puppetlabs/activemq',
        path    => '/opt/puppet/bin:/usr/kerberos/sbin:/usr/kerberos/bin:/usr/local/sbin:/usr/local/bin:/sbin:/bin:/usr/sbin:/usr/bin',
        creates => '/etc/puppetlabs/activemq/broker.ts',
        before  => [ File['/etc/puppetlabs/activemq/broker.ts'] ],
      }
      file { '/etc/puppetlabs/activemq/broker.ts':
        ensure => file,
        owner   => '0',
        group   => 'pe-activemq',
        mode    => '0640',
        notify => [ Service['pe-activemq'] ],
      }
    }
  }

  # Configure the AES keys on each mcollective server (Note, these are not
  # actually used as SSL certificates, they're just used for their public and
  # private keys if AES security is enabled.)
  file { '/etc/puppetlabs/mcollective/ssl':
    ensure => directory,
    owner  => '0',
    group  => '0',
    mode   => '0755',
    notify => Service['mcollective'],
  }
  # The use of the selector is because we can't use the file() function in a single
  # run on the puppet master.  The file won't exist until the catalog is applied.
  file { 'mcollective-public.pem':
    path    => '/etc/puppetlabs/mcollective/ssl/mcollective-public.pem',
    ensure  => file,
    source  => $is_puppetmaster ? {
      true  => '/etc/puppetlabs/puppet/ssl/public_keys/pe-internal-mcollective-servers.pem',
      false => undef,
    },
    content => $is_puppetmaster ? {
      true  => undef,
      false => file('/etc/puppetlabs/puppet/ssl/public_keys/pe-internal-mcollective-servers.pem'),
    },
    owner   => '0',
    group   => '0',
    mode    => '0644',
    notify  => Service['mcollective'],
  }
  # The use of the selector is because we can't use the file() function in a single
  # run on the puppet master.  The file won't exist until the catalog is applied.
  file { 'mcollective-private.pem':
    path    => '/etc/puppetlabs/mcollective/ssl/mcollective-private.pem',
    ensure  => file,
    source  => $is_puppetmaster ? {
      true  => '/etc/puppetlabs/puppet/ssl/private_keys/pe-internal-mcollective-servers.pem',
      false => undef,
    },
    content => $is_puppetmaster ? {
      true  => undef,
      false => file('/etc/puppetlabs/puppet/ssl/private_keys/pe-internal-mcollective-servers.pem'),
    },
    owner   => '0',
    group   => '0',
    mode    => '0600',
    notify  => Service['mcollective'],
  }
  # The use of the selector is because we can't use the file() function in a single
  # run on the puppet master.  The file won't exist until the catalog is applied.
  file { 'mcollective-cert.pem':
    path    => '/etc/puppetlabs/mcollective/ssl/mcollective-cert.pem',
    ensure  => file,
    source  => $is_puppetmaster ? {
      true  => '/etc/puppetlabs/puppet/ssl/certs/pe-internal-mcollective-servers.pem',
      false => undef,
    },
    content => $is_puppetmaster ? {
      true  => undef,
      false => file('/etc/puppetlabs/puppet/ssl/certs/pe-internal-mcollective-servers.pem'),
    },
    owner   => '0',
    group   => '0',
    mode    => '0644',
    notify  => Service['mcollective'],
  }
  file { '/etc/puppetlabs/mcollective/ssl/clients':
    ensure => directory,
    owner  => '0',
    group  => '0',
    mode   => '0755',
  }
  file { 'peadmin-public.pem':
    path    => '/etc/puppetlabs/mcollective/ssl/clients/peadmin-public.pem',
    source  => $is_puppetmaster ? {
      true  => '/etc/puppetlabs/puppet/ssl/public_keys/pe-internal-peadmin-mcollective-client.pem',
      false => undef,
    },
    content => $is_puppetmaster ? {
      true  => undef,
      false => file('/etc/puppetlabs/puppet/ssl/public_keys/pe-internal-peadmin-mcollective-client.pem'),
    },
    ensure  => 'file',
    owner   => '0',
    group   => '0',
    mode    => '0644',
    notify  => Service['mcollective'],
  }
  file { 'puppet-dashboard-public.pem':
    path    => '/etc/puppetlabs/mcollective/ssl/clients/puppet-dashboard-public.pem',
    source  => $is_puppetmaster ? {
      true  => '/etc/puppetlabs/puppet/ssl/public_keys/pe-internal-puppet-console-mcollective-client.pem',
      false => undef,
    },
    content => $is_puppetmaster ? {
      true  => undef,
      false => file('/etc/puppetlabs/puppet/ssl/public_keys/pe-internal-puppet-console-mcollective-client.pem'),
    },
    ensure  => 'file',
    owner   => '0',
    group   => '0',
    mode    => '0644',
    notify  => Service['mcollective'],
  }
  # All servers need the shared public key to decrypt and encrypt
  # registration messages.
  file { '/etc/puppetlabs/mcollective/ssl/clients/mcollective-public.pem':
    ensure  => file,
    source  => '/etc/puppetlabs/mcollective/ssl/mcollective-public.pem',
    owner   => '0',
    group   => '0',
    mode    => '0644',
    notify  => Service['mcollective'],
    require => File['mcollective-public.pem'],
  }

  # Manage the MCollective server configuration
  file { '/etc/puppetlabs/mcollective/server.cfg':
    ensure  => file,
    content => template("${module_name}/server.cfg.erb"),
    owner   => '0',
    group   => '0',
    mode    => '0600',
    notify  => Service['mcollective'],
  }
  service { 'mcollective':
    ensure     => running,
    name       => 'pe-mcollective',
    enable     => true,
    hasstatus  => true,
    hasrestart => true,
  }

  # The Puppet Master is the only system to get the peadmin user account
  # and user-specific encryption keys.
  if $is_puppetmaster {
    pe_accounts::user { 'peadmin':
      ensure   => present,
      password => '!!',
      home     => '/var/lib/peadmin',
    }
    file { '/var/lib/peadmin/.mcollective':
      ensure  => file,
      content => template('pe_mcollective/client_peadmin.cfg.erb'),
      owner   => 'peadmin',
      group   => 'peadmin',
      mode    => '0600',
      require => Pe_accounts::User['peadmin'],
    }
    file { '/etc/puppetlabs/mcollective/client.cfg':
      ensure  => file,
      content => template('pe_mcollective/client.cfg.erb'),
      owner   => '0',
      group   => '0',
      mode    => '0600',
    }
    file { '/var/lib/peadmin/.mcollective.d':
      ensure  => directory,
      owner   => 'peadmin',
      group   => 'peadmin',
      mode    => '0700',
      require => Pe_accounts::User['peadmin'],
    }
    file { '/var/lib/peadmin/.mcollective.d/peadmin-private.pem':
      ensure  => file,
      source  => '/etc/puppetlabs/puppet/ssl/private_keys/pe-internal-peadmin-mcollective-client.pem',
      owner   => 'peadmin',
      group   => 'peadmin',
      mode    => '0600',
      require => [ Pe_accounts::User['peadmin'], Exec['mcollective-client-cert'] ],
    }
    file { '/var/lib/peadmin/.mcollective.d/peadmin-public.pem':
      ensure  => file,
      source  => '/etc/puppetlabs/puppet/ssl/public_keys/pe-internal-peadmin-mcollective-client.pem',
      owner   => 'peadmin',
      group   => 'peadmin',
      mode    => '0644',
      require => [ Pe_accounts::User['peadmin'], Exec['mcollective-client-cert'] ],
    }
    file { '/var/lib/peadmin/.mcollective.d/peadmin-cert.pem':
      ensure  => file,
      source  => '/etc/puppetlabs/puppet/ssl/certs/pe-internal-peadmin-mcollective-client.pem',
      owner   => 'peadmin',
      group   => 'peadmin',
      mode    => '0644',
      require => [ Pe_accounts::User['peadmin'], Exec['mcollective-client-cert'] ],
    }
    # Because the accounts module is managing the .bashrc, we use
    # .bashrc.custom, which is included by default in the managed .bashrc
    file { '/var/lib/peadmin/.bashrc.custom':
      ensure  => file,
      content => template("${module_name}/bashrc_custom.erb"),
      owner   => 'peadmin',
      group   => 'peadmin',
      mode    => '0644',
      require => Pe_accounts::User['peadmin'],
    }
  }

  if $is_puppetconsole {
    # #9694 - Manage the encryption keys for the puppet-dashboard user as well.
    pe_accounts::user { 'puppet-dashboard':
      ensure   => present,
      password => '!!',
      home     => '/opt/puppet/share/puppet-dashboard',
    }
    # The home directory must be world-readable for the Dashboard to work
    # in Apache.  This resource is declared in the accounts::user defined
    # type.
    File <| title == '/opt/puppet/share/puppet-dashboard' |> {
      mode => '0755'
    }
    # Manage the path correctly
    file { '/opt/puppet/share/puppet-dashboard/.bashrc.custom':
      ensure  => file,
      content => template("${module_name}/bashrc_custom.erb"),
      owner   => 'puppet-dashboard',
      group   => 'puppet-dashboard',
      mode    => '0600',
      require => Pe_accounts::User['puppet-dashboard'],
    }
    file { '/opt/puppet/share/puppet-dashboard/.mcollective':
      ensure  => file,
      content => template('pe_mcollective/client_puppet_dashboard.cfg.erb'),
      owner   => 'puppet-dashboard',
      group   => 'puppet-dashboard',
      mode    => '0600',
      require => Pe_accounts::User['puppet-dashboard'],
    }
    file { '/opt/puppet/share/puppet-dashboard/.mcollective.d':
      ensure  => directory,
      owner   => 'puppet-dashboard',
      group   => 'puppet-dashboard',
      mode    => '0700',
      require => Pe_accounts::User['puppet-dashboard'],
    }
    # Master+console installs use source with a local file path for the content
    # console only installs use content with a file function to deliver in-band
    # Our basic premise is that master installs are completed before any console install.
    file { '/opt/puppet/share/puppet-dashboard/.mcollective.d/puppet-dashboard-private.pem':
      ensure  => file,
      content => $is_puppetmaster ? {
        true => undef,
        false => file('/etc/puppetlabs/puppet/ssl/private_keys/pe-internal-puppet-console-mcollective-client.pem'),
      },
      source  => $is_puppetmaster? {
        true  => '/etc/puppetlabs/puppet/ssl/private_keys/pe-internal-puppet-console-mcollective-client.pem',
        false => undef,
      },
      owner   => 'puppet-dashboard',
      group   => 'puppet-dashboard',
      mode    => '0600',
      require => $is_puppetmaster ? {
        true  => [ Pe_accounts::User['puppet-dashboard'], Exec['puppet-dashboard-client-cert'] ],
        false => Pe_accounts::User['puppet-dashboard'],
      },
    }
    file { '/opt/puppet/share/puppet-dashboard/.mcollective.d/puppet-dashboard-public.pem':
      ensure  => file,
      content => $is_puppetmaster ? {
        true  => undef,
        false => file('/etc/puppetlabs/puppet/ssl/public_keys/pe-internal-puppet-console-mcollective-client.pem'),
      },
      source  => $is_puppetmaster ? {
        true  => '/etc/puppetlabs/puppet/ssl/public_keys/pe-internal-puppet-console-mcollective-client.pem',
        false => undef,
      },
      owner   => 'puppet-dashboard',
      group   => 'puppet-dashboard',
      mode    => '0644',
      require => $is_puppetmaster ? {
        true  => [ Pe_accounts::User['puppet-dashboard'], Exec['puppet-dashboard-client-cert'] ],
        false => Pe_accounts::User['puppet-dashboard'],
      },
    }
    file { '/opt/puppet/share/puppet-dashboard/.mcollective.d/puppet-dashboard-cert.pem':
      ensure  => file,
      content => $is_puppetmaster ? {
        true  => undef,
        false => file('/etc/puppetlabs/puppet/ssl/certs/pe-internal-puppet-console-mcollective-client.pem'),
      },
      source  => $is_puppetmaster ? {
        true  => '/etc/puppetlabs/puppet/ssl/certs/pe-internal-puppet-console-mcollective-client.pem',
        false => undef,
      },
      owner   => 'puppet-dashboard',
      group   => 'puppet-dashboard',
      mode    => '0644',
      require => $is_puppetmaster ? {
        true  => [ Pe_accounts::User['puppet-dashboard'], Exec['puppet-dashboard-client-cert'] ],
        false => Pe_accounts::User['puppet-dashboard'],
      },
    }
  }

  # (#8826) Manage the plugins for MCollective in Puppet Enterprise
  # The anchors are to gurantee resource containment for the module
  anchor { 'pe_mcollective::begin': }
  class { 'pe_mcollective::plugins':
    require => Anchor['pe_mcollective::begin'],
    before  => Anchor['pe_mcollective::end'],
  }
  # (#9045) Update facter facts on disk periodically using a cron job.
  # This is a separate class because I'm anticipating platform oddities
  # with cron on different platforms.
  class { 'pe_mcollective::metadata':
    require => Anchor['pe_mcollective::begin'],
    before  => Anchor['pe_mcollective::end'],
  }
  anchor { 'pe_mcollective::end': }
}
